#!/usr/bin/python

"CERN Printer Wizard for CEL 3"
"author: Marek Misiowiec"
"e-mail: marek.misiowiec@cern.ch"
"version: 1.0"
"date: February 2004"

import os
import re
from sys import exit
from string import split, strip, rstrip

curDir = os.curdir + os.sep
parDir = os.pardir + os.sep
sepDir = os.sep

ftpServer = "cernsoft01.cern.ch"
ftpServerPath = "cernprnt/data"

filesPath = "/usr/share/printconf/cern"
buildingsFileName = "Buildings.txt"
buildingsFileLocal = filesPath + sepDir + buildingsFileName
printerListFileName = "PrinterList.txt"
printerListFileLocal = filesPath + sepDir + printerListFileName
printerTypesFileName = "PrinterTypes.txt"
printerTypesFileLocal = filesPath + sepDir + printerTypesFileName
cerndefaultprinter = filesPath + sepDir + "DefaultPrinter"

TIMEOUT = 5

ANY_TYPE = "<ALL>"
ANY_BUILDING = "<all>"

MAX_MODEL_LETTERS = 35 # maximum number of letters to be displayed on printers list

NAME_CHEATER = "cpw_"
NAME_CHEATER_LEN = 4

def filterNameToView (name_):
#    print "filterTo"
#    return name_
    if re.match(NAME_CHEATER, name_):
        return name_[NAME_CHEATER_LEN:]        
    else:
        return name_

def filterNameFromView (name_):
#    print "filterFrom"
#    return name_
    if re.match('^[0-9]', name_):
        return NAME_CHEATER + name_
    else:
        return name_ 	


class Printer:
  def init (self):
    pass
  def initialize (self, address_, building_, type_, name_):
    self.address = address_
    self.building = building_
    self.printtype = type_
    self.name = name_


class PrintersInput:
  "Defines operation related to input data. Among others, \
  It reads files from ftp server, parses their contents and \
  creates appropriate dictionaries."

  def __init__ (self):
    self.buildingsDict = {}
    self.printerTypesDict = {}
    self.printersDict = {}


  def readFile (self, filename):
    "Opens file filename and reads its lines into an array. \
    If there's no file or there's been an error during opening, \
    it returns None, in other case it returns the array." 
    try:
      fileHandle = open (filename, "r")
      fileLines = fileHandle.readlines()
      fileHandle.close()
    except IOError, text:
      print text
      return None
    return fileLines

  def lastModification(self):
    from os import lstat
    from time import ctime
    t = str(ctime(lstat(printerListFileLocal)[-1]))
#    print t
    return t    
  

  def splitLines (self, fileLines):
    "Splits lines from one data source (fileLines is a list of lines read \
     from a text file) using @ as separator"
    from string import split, rstrip
    array = []
    for line in fileLines:
      newLine = split(rstrip(line,' \t\r\n'), '@')
      array.append(newLine)
    return array


  def createBuildingsDict(self):
    fileLines = self.readFile(buildingsFileLocal)

    if fileLines == None:
      return None

    newLines = self.splitLines(fileLines)

    for line in newLines:
      for lline in line:
        lline = str(lline)
        if self.buildingsDict.has_key(lline):
          print "Inconsistent data (buildings)!"
          exit(-1)
        else:
          self.buildingsDict[lline] = []
    return self.buildingsDict


  def createPrinterTypesDict(self):
    fileLines = self.readFile(printerTypesFileLocal)

    if fileLines == None:
      return None

    newLines = self.splitLines(fileLines)

    for line in newLines:
      for lline in line:
        lline = str(lline)
        if self.printerTypesDict.has_key(lline):
          print "Inconsistent data (printer types)!"
          exit(-1)
        else:
          self.printerTypesDict[lline] = lline
    return self.printerTypesDict


  def createPrintersDict (self):
    fileLines = self.readFile(printerListFileLocal)

    if fileLines == None:
      return None

    newLines = self.splitLines(fileLines)

#    print "printer lines: " + str(len(newLines))

    for line in newLines:
      address_ = str(line[0])    
      if strip(address_) == "":
        continue
      if self.printersDict.has_key(address_):
        print "I saw \'%s\' before - repeating data (printers) skipped!" % address_
        continue
      else:
        name_ = str(line[9])
        name_ = name_[0:MAX_MODEL_LETTERS]
        building_ = str(line[8])
        type_ = str(line[7])
        if (not self.buildingsDict.has_key(building_)) or (not self.printerTypesDict.has_key(type_)):
          print "Inconsistent data (printers)!"
          print address_ + " " + building_ + " " + type_
          exit(-1)
        else:
          newPrinter = Printer()
          newPrinter.initialize(address_, building_, type_, name_)
          self.printersDict[address_] = newPrinter
          self.buildingsDict.get(building_).append(newPrinter)

#    print str(self.buildingsDict)
    return self.printersDict


  def handleSigalrm (self, *args):
    from socket import error
    raise error


  def ftpDownloadFiles(self, hostName, filePath, fileNameList, fileNameLocals):
    "Downloads files listed in fileNameList from directory filePath on a server hostName, \
     using anonymous login. Files are stored in a current directory with relevant names."
    CHUNK = 16384
    from ftplib import FTP
    from string import split
    import socket, signal

    # downloading with timeout:
    try:
      signal.signal(signal.SIGALRM, self.handleSigalrm)
      signal.alarm(TIMEOUT) # activate an alarm

      ftp = FTP(hostName)
#      ftp.connect("anonymous", "guest@anonymous.guest")
      ftp.login() # anonymous login
      ftp.cwd(filePath)

      i = 0
      for fileName in fileNameList:
#        print "file: " + fileName + " - size: " + str(ftp.size(fileName))
        f = open (fileNameLocals[i], 'w', 0444)
#        print "there"
        ftp.retrbinary ('RETR ' + fileName, lambda x: f.write (x), CHUNK)
        f.close()
        i += 1

      ftp.quit()

      signal.alarm(0) # disable the alarm
      return 1

    except socket.error:
      print "Socket error "
#      if ftp:
#         ftp.close()
      return 0
    except IOError, name:
      print "IOError: " + str(name)
#      if ftp:
#         ftp.close()
      return 0
    except:
      print "Unexpected error "
      return 0


class PrintersData:

  def __init__ (self):
    self.buildingsDict = {}
    self.printerTypesDict = {}
    self.printersDict = {}

  def refresh (self):
    self.printersInput = PrintersInput()

#    print "Downloading files...	",
    self.downloaded = self.printersInput.ftpDownloadFiles(ftpServer, ftpServerPath, \
													[buildingsFileName, printerListFileName, printerTypesFileName], \
													[buildingsFileLocal, printerListFileLocal, printerTypesFileLocal],)
#    if self.downloaded == 1:
#      print "done."
#    else:
#      print "failed. Old data used."

#    print "Parsing input data...	", 
    self.buildingsDict = self.printersInput.createBuildingsDict()
    self.printerTypesDict = self.printersInput.createPrinterTypesDict()
    self.printersDict = self.printersInput.createPrintersDict()

    if self.buildingsDict == None or self.printerTypesDict == None or self.printersDict == None:
      print "failed. No data - application aborted."
      sys.exit(-1)
#    else:
#      print "done."

    self.lastModified = self.printersInput.lastModification()


# map (lambda x: x.type_name, self.queue_types)

  def extract (self, selectedBuildings, selectedPrinterTypes):

    anyBuilding = 0
    for i in selectedBuildings:
      if i == ANY_BUILDING:
        anyBuilding = 1

    anyType = 0
    for i in selectedPrinterTypes:
      if i == ANY_TYPE:
        anyType = 1

    selectedPrinterTypesDict = {}

    if not anyType:
      for i in selectedPrinterTypes:
        selectedPrinterTypesDict [i] = i

    resultArray = []


    if anyBuilding:
      for array in self.buildingsDict.values():
        if anyType:
          resultArray.extend(array)
        else:
          for printer in array:
            if selectedPrinterTypesDict.has_key(printer.printtype):
#              print printer.printtype
              resultArray.append(printer)
    else:
      for i in selectedBuildings:
        array = self.buildingsDict.get(str(i))
#        print "all: " + str(len(array))
        if anyType:
          resultArray.extend(array)
        else:
          for printer in array:
            if selectedPrinterTypesDict.has_key(printer.printtype):
#             print printer.printtype
              resultArray.append(printer)

#    print "\nsel: " + str(len(resultArray))
    return resultArray
		      
def main():
  printersData = PrintersData()
  printersData.refresh()

if __name__ == '__main__':
  main ()
